"use client";
import { GatsbyVideo } from "@bond-london/gatsby-transformer-video";
import React, {
  CSSProperties,
  useCallback,
  useState,
  VideoHTMLAttributes,
} from "react";
import { calculateCropDetails } from "../utils";
import { IBondSimpleVideo } from "./BondSimpleVideo";
import { ICmsVideo } from "./BondVideo";
import ReactPlayer from "react-player/lazy";
import { VideoControls } from "./VideoControls";

export type IBondExternalVideo = IBondSimpleVideo & {
  external: string;
  loop?: boolean;
};

export function convertCmsVideoToBondExternalVideo(
  cms: ICmsVideo
): IBondExternalVideo {
  const preview = cms.preview.localFile?.childGatsbyVideo?.transformed;

  if (!preview) {
    throw new Error("No preview found");
  }

  const external = cms.external || undefined;

  if (!external) {
    throw new Error("No external video found");
  }

  const posterFile = cms.poster?.localFile?.publicURL || undefined;

  return {
    videoData: posterFile ? { ...preview, poster: posterFile } : preview,
    loop: cms.loop || undefined,
    external,
    dontCrop: cms.dontCrop,
    verticalCropPosition: cms.verticalCropPosition,
    horizontalCropPosition: cms.horizontalCropPosition,
  };
}

export const BondExternalVideo: React.FC<
  {
    video: IBondExternalVideo;
    autoLoad?: boolean;
    videoClassName?: string;
    videoStyle?: CSSProperties;
    noPoster?: boolean;
    playButton?: React.FC<{ playVideo?: () => void }>;
    pauseButton?: React.FC<{ pauseVideo?: () => void }>;
    muteButton?: React.FC<{ muteVideo?: () => void }>;
    unmuteButton?: React.FC<{ unmuteVideo?: () => void }>;
    showAudioControls?: boolean;
  } & Omit<
    VideoHTMLAttributes<HTMLVideoElement>,
    "poster" | "objectFit" | "objectPosition"
  >
> = props => {
  const [isPlaying, setIsPlaying] = useState(false);
  const [isMuted, setIsMuted] = useState(false);
  const [previewHasStarted, setPreviewHasStarted] = useState(false);
  const [fullRequested, setFullRequested] = useState(false);
  const [fullHasLoaded, setFullHasLoaded] = useState(false);
  const [fullHasStarted, setFullHasStarted] = useState(false);

  const onPreviewHasStarted = useCallback(() => setPreviewHasStarted(true), []);
  const onFullRequested = useCallback(() => setFullRequested(true), []);
  const onFullLoaded = useCallback(() => setFullHasLoaded(true), []);
  const onFullStarted = useCallback(() => setFullHasStarted(true), []);
  const playVideo = useCallback(() => setIsPlaying(true), []);
  const pauseVideo = useCallback(() => setIsPlaying(false), []);
  const muteVideo = useCallback(() => setIsMuted(true), []);
  const unmuteVideo = useCallback(() => setIsMuted(false), []);

  const {
    video,
    playButton,
    pauseButton,
    muteButton,
    unmuteButton,
    showAudioControls,
    loop,
    ...videoProps
  } = props;
  const {
    dontCrop,
    horizontalCropPosition,
    verticalCropPosition,
    external,
    videoData,
    loop: videoLoop,
  } = video;
  const { objectFit, objectPosition } = calculateCropDetails({
    dontCrop,
    horizontalCropPosition,
    verticalCropPosition,
  });

  const loadFull = (props.autoLoad && previewHasStarted) || fullRequested;
  const showFullRequest = !loadFull && !props.autoLoad && !fullRequested;
  const fullShouldPlay = isPlaying || (fullRequested && !fullHasStarted);

  return (
    <GatsbyVideo
      {...videoProps}
      loop={true}
      video={videoData}
      onTimeUpdate={!previewHasStarted ? onPreviewHasStarted : undefined}
      pause={fullHasLoaded ? true : undefined}
      objectFit={objectFit}
      objectPosition={objectPosition}
      videoClassName={fullHasLoaded ? "opacity-0" : "opacity-100"}
    >
      {showFullRequest && (
        <VideoControls playVideo={onFullRequested} playButton={playButton} />
      )}
      {loadFull && (
        <ReactPlayer
          url={external}
          style={{ objectFit, objectPosition, opacity: fullHasStarted ? 1 : 0 }}
          className="inside"
          onReady={onFullLoaded}
          onPlay={onFullStarted}
          width="100%"
          height="100%"
          controls={false}
          playing={fullShouldPlay}
          muted={isMuted}
          loop={loop || videoLoop}
          playsinline={videoProps.playsInline}
        />
      )}
      {fullHasLoaded && (
        <VideoControls
          isPlaying={isPlaying}
          isMuted={isMuted}
          playVideo={playVideo}
          pauseVideo={pauseVideo}
          muteVideo={muteVideo}
          unmuteVideo={unmuteVideo}
          playButton={playButton}
          pauseButton={pauseButton}
          muteButton={muteButton}
          unmuteButton={unmuteButton}
          showAudioControls={showAudioControls}
        />
      )}
    </GatsbyVideo>
  );
};
